
#include <hdf/hdf5.h>

int main(int, char const *[])
{
	int ret = 0;


	hid_t fid;
	if ((fid = H5Fopen("test.h5",H5F_ACC_RDWR,H5P_DEFAULT)) < 0)
		return -1;
	/*
	bizarre... it works if I create my own file, but not with the test.h5 I created 
	with HDFView
	if ((fid = H5Fcreate("test.h5",H5F_ACC_EXCL,H5P_DEFAULT,H5P_DEFAULT)) < 0)
		return -1;
	*/

	// we'll be creating a dataset for ints
	hid_t dtype_id =  H5Tcopy(H5T_NATIVE_INT);
	// create 1D dataset with 10 initial size, and unlimited max size
	const unsigned int FSPACE_RANK = 1;
	hsize_t 
		fspace_dims[FSPACE_RANK],
		fspace_maxdims[FSPACE_RANK];
	fspace_dims[0] = 10;
	fspace_maxdims[0] = H5S_UNLIMITED;
	hid_t fspace_id = H5Screate_simple(FSPACE_RANK,fspace_dims, fspace_maxdims);

	// dataset written in chunks of 100, compressed w/ zlib, and early alloc time
	hsize_t fspace_chunkdims[FSPACE_RANK];
	fspace_chunkdims[0] = 100;
	hid_t dcpl = H5Pcreate(H5P_DATASET_CREATE);
	H5Pset_chunk(dcpl,FSPACE_RANK,fspace_chunkdims);
	H5Pset_deflate(dcpl,6);
	H5Pset_alloc_time(dcpl,H5D_ALLOC_TIME_EARLY);

	hid_t lcpl = H5Pcreate (H5P_LINK_CREATE);
	H5Pset_create_intermediate_group(lcpl,1);

	hid_t grp_id;

	hid_t dset_id;

	if ((grp_id = H5Gcreate(fid,"/newGroup",H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT)) < 0)
	{
		printf("Problem creating group\n");
	}

	// get an assert if I try to create dataset /newGroup/newDataset
	// but creating dataset beneath root dir or pre-existing groups both work OK
	//
	//the following assert :
	//Assertion failed: elem, file ..\..\..\src\H5FL.c, line 1876
	//Assertion failed: H5F_sfile_head_g == ((void *)0), file ..\..\..\src\H5Fsfile.c, line 65

	if ((dset_id = H5Dcreate(fid,"/newGroup/newDataset",dtype_id,fspace_id,lcpl,dcpl,H5P_DEFAULT)) < 0)
	{
		printf("Problem creating dataset\n");
	}


	if (grp_id >= 0)
		H5Gclose(grp_id);
	if (dset_id >= 0)
		H5Dclose(dset_id);

	if (dtype_id >= 0)
		H5Tclose(dtype_id);
	if (dcpl >= 0)
		H5Pclose(dcpl);
	if (lcpl >= 0)
		H5Pclose(lcpl);
	if (fid >= 0)
		H5Fclose(fid);

	return ret;
}
